"""
Unit testing.
"""
import unittest
from .. import gen_sympy
from .. import gen_sympy2
import os
import subprocess

TEST_DIR = os.path.dirname(os.path.realpath(__file__))

class Test(unittest.TestCase):
    "Testing"

    def test_bouncing_ball(self):
        "Test if bouncing ball simulates"
        #pylint: disable=no-self-use
        src = os.path.join(TEST_DIR, 'BouncingBall.mo')
        out = os.path.join(TEST_DIR, 'BouncingBall_pymola.py')
        gen_sympy.main(argv=[src, out])
        cmd = 'python {out:s}'.format(**locals())
        proc = subprocess.Popen(cmd.split())
        proc.communicate()

    def test_bouncing_ball2(self):
        "Test if bouncing ball simulates with visitor version"
        #pylint: disable=no-self-use
        src = os.path.join(TEST_DIR, 'BouncingBall.mo')
        out = os.path.join(TEST_DIR, 'BouncingBall2_pymola.py')
        gen_sympy2.main(argv=[src, out])
        # cmd = 'python {out:s}'.format(**locals())
        # proc = subprocess.Popen(cmd.split())
        # proc.communicate()

    def test_estimator(self):
        "Test if estimator simulates"
        #pylint: disable=no-self-use
        src = os.path.join(TEST_DIR, 'Estimator.mo')
        out = os.path.join(TEST_DIR, 'Estimator_pymola.py')
        gen_sympy.main(argv=[src, out])
        cmd = 'python {out:s}'.format(**locals())
        proc = subprocess.Popen(cmd.split())
        proc.communicate()






# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '0.0.6'
version = '0.0.6'
full_version = '0.0.6'
git_revision = 'f9f544fbfbb6913ac159a8b57777b133ccec6f66'
release = True

if not release:
    version = full_version



#!/usr/bin/env python
"""
Modelica compiler.
"""
from __future__ import print_function
import sys
import antlr4
import antlr4.Parser
from generated.ModelicaLexer import ModelicaLexer
from generated.ModelicaParser import ModelicaParser
from generated.ModelicaListener import ModelicaListener
import argparse
import jinja2

#pylint: disable=invalid-name, no-self-use, missing-docstring, unused-variable, protected-access
#pylint: disable=too-many-public-methods

class SympyPrinter(ModelicaListener):

    #-------------------------------------------------------------------------
    # Setup
    #-------------------------------------------------------------------------

    def __init__(self, parser, trace):
        """
        Constructor
        """
        self._val_dict = {}
        self.result = None
        self._parser = parser
        self._trace = trace
        self.indent = "            "

    @staticmethod
    def print_indented(ldr, s):
        if s is not None:
            for line in s.split('\n'):
                print(ldr, line)

    def setValue(self, ctx, val):
        """
        Sets tree values.
        """
        if ctx.depth() == 1:
            self.result = val
        else:
            self._val_dict[ctx] = val

    def getValue(self, ctx):
        """
        Gets tree values.
        """
        assert ctx is not None
        if ctx.depth() == 1:
            return self.result
        else:
            try:
                return self._val_dict[ctx]
            except KeyError:
                return None

    def enterEveryRule(self, ctx):
        if self._trace:
            ldr = " "*(ctx.depth()-1)
            rule = self._parser.ruleNames[ctx.getRuleIndex()]

            print(ldr, rule + "{")
            in_str = ctx.getText()
            if in_str > 0:
                print(ldr, "==============================")
                print(ldr, "INPUT\n")
                self.print_indented(ldr, ctx.getText())
                print(ldr, "==============================\n")

    def visitTerminal(self, node):
        pass

    def visitErrorNode(self, node):
        pass

    def exitEveryRule(self, ctx):
        rule = self._parser.ruleNames[ctx.getRuleIndex()]
        if self._trace:
            ldr = " "*ctx.depth()
            lines = None
            try:
                lines = self.getValue(ctx)
            except KeyError as e:
                pass

            if lines is not None:
                print(ldr, "==============================")
                print(ldr, "OUTPUT\n")
                self.print_indented(ldr, lines)
                print(ldr, "==============================\n")

            print(ldr + '} //' + rule + '\n')
        if self.getValue(ctx) is None:
            raise RuntimeError("no value set for {:s}:\ninput:\n{:s}".format(rule, ctx.getText()))

    #-------------------------------------------------------------------------
    # Top Level
    #-------------------------------------------------------------------------

    def exitStored_definition(self, ctx):
        composition = self.getValue(ctx.class_definition()[0]\
                .class_specifier().composition())
        self.result = """
#############################################################################
# Automatically generated by pymola

from __future__ import print_function
import sympy
import sympy.physics.mechanics as mech
sympy.init_printing()
mech.init_vprinting()
import scipy.integrate
import pylab as pl

#pylint: disable=too-few-public-methods, too-many-locals, invalid-name, no-member

class Model(object):
    \"\"\"
    Modelica Model.
    \"\"\"

    def __init__(self):
        \"\"\"
        Constructor.
        \"\"\"

        self.t = sympy.symbols('t')

        {composition:s}

        self.x = sympy.Matrix(self.x)
        self.x_dot = self.x.diff(self.t)

        self.sol = sympy.solve(self.eqs, self.x_dot)

        self.f = sympy.Matrix([self.sol[xi] for xi in self.x_dot])
        print('x:', self.x)
        print('f:', self.f)

        self.p_vect = [locals()[key] for key in self.p_dict.keys()]
        self.p0 = [self.p_dict[key] for key in self.p_dict.keys()]

        print('p:', self.p_vect)

        self.f_lam = sympy.lambdify((self.t, self.x, self.p_vect), self.f)

        self.x0 = [self.x0_dict[key] for key in self.x0_dict.keys()]

    def simulate(self, tf=30, dt=0.001, show=False):
        \"\"\"
        Simulation function.
        \"\"\"

        sim = scipy.integrate.ode(self.f_lam)
        sim.set_initial_value(self.x0, 0)
        sim.set_f_params(self.p0)

        data = {{
            'x': [],
            't': [],
        }}

        while  sim.t < tf:
            sim.integrate(sim.t + dt)
            t = sim.t

            # TODO replace hardcoded when statement
            # below
            #velocity = sim.y[0]
            #height = sim.y[1]
            #c = self.p0[0]
            #if velocity < 0 and height < 0:
            #    velocity = -c*velocity
            #    height = 0
            #    sim.set_initial_value([velocity, height], t)

            # data['x'] += [[velocity, height]]
            data['x'] += [sim.y]
            data['t'] += [t]

        pl.plot(data['t'], data['x'])
        if show:
            pl.show()

        return data

if __name__ == "__main__":
    model = Model()
    model.simulate()

#############################################################################
""".format(**locals())

    def exitComposition(self, ctx):
        decl = self.getValue(ctx.element_list()[0])
        equations = self.getValue(ctx.equation_section()[0])
        data = locals()
        data['walker'] = self
        data.pop('self')
        self.setValue(ctx, jinja2.Template("""
        {%- set params=decl.parameters -%}
        {%- set dsyms=decl.dynamic_symbols -%}
        {%- set start_values=decl.start_values -%}

        {% if params|length > 0 %}
        # symbols
        {{params|join(', ') }} = \\
            sympy.symbols('{{params|join(', ')}}')
        {% endif %}

        # dynamic symbols
        {{dsyms|join(', ')}} = \\
            mech.dynamicsymbols('{{dsyms|join(', ')}}')

        # parameters
        self.p_dict = {
        {%- for key in params.keys() %}
            '{{key}}': {{params[key]}},
        {%- endfor %}
        }

        # initial sate
        self.x0_dict = {
        {%- for key in dsyms.keys() %}
            '{{key}}': {{dsyms[key].start}},
        {%- endfor %}
        }

        # state space
        self.x = sympy.Matrix([
            {{dsyms|join(', ')}}
        ])
{{equations}}
""").render(**data))

    def exitElement_list(self, ctx):
        d = self.getValue(ctx.element()[0])
        for i in range(1, len(ctx.element())):
            element = ctx.element()[i]
            for key in d.keys():
                d[key].update(self.getValue(element)[key])
        self.setValue(ctx, d)
        print("dict", d)

    def exitElement(self, ctx):
        self.setValue(ctx, self.getValue(ctx.component_clause()))

    def exitEquation_section(self, ctx):
        str_eq = "self.eqs = ["
        str_when = ""
        for eq in ctx.equation():
            data = self.getValue(eq)
            if len(data) <= 0:
                continue
            print("eq type:", type(eq.equation_options()))
            if isinstance(eq.equation_options(), ModelicaParser.Equation_simpleContext):
                str_eq += "\n{self.indent:s}{data:s},".format(**locals())
                print("SIMPLE")
            elif isinstance(eq.equation_options(), ModelicaParser.Equation_whenContext):
                str_when += "\n{self.indent:s}{data:s}".format(**locals())
            else:
                raise IOError('equation type not supported yet')
        str_eq += "\n{self.indent:s}]\n".format(**locals())
        str_when += ""
        self.setValue(ctx, """
        # equations
        {str_eq:s}

        # when equations
{str_when:s}
""".format(**locals()))

    def exitComponent_clause(self, ctx):
        s = ""
        parameters = {}
        dynamic_symbols = {}

        if ctx.type_prefix().getText() == 'parameter':
            # store all parameters
            for comp in ctx.component_list().component_declaration():
                name = comp.declaration().IDENT().getText()
                val = comp.declaration().modification().expression().getText()
                parameters[name] = val
        else:
            # store all variables
            for comp in ctx.component_list().component_declaration():
                name = comp.declaration().IDENT().getText()
                dynamic_symbols[name] = {'start': 0}
                try:
                    mod = comp.declaration().modification().class_modification()
                    arg = mod.argument_list().argument()[0]
                    emod = arg.element_modification_or_replaceable().element_modification()
                    if emod.name().getText() == 'start':
                        val = emod.modification().expression().getText()
                        dynamic_symbols[name]['start'] = val
                except AttributeError:
                    pass

        self.setValue(ctx, {
            'parameters': parameters,
            'dynamic_symbols': dynamic_symbols,
            })

    def exitClass_prefixes(self, ctx):
        # don't care about prefixes for now
        self.setValue(ctx, '')

    def exitName(self, ctx):
        self.setValue(ctx, ctx.getText())

    def exitType_specifier(self, ctx):
        self.setValue(ctx, self.getValue(ctx.name()))

    def exitModification_class(self, ctx):
        mod = self.getValue(ctx.class_modification())
        if ctx.expression() is not None:
            expr = self.getValue(ctx.expression())
            self.setValue(ctx, "{:s} = {:s}".format(mod, expr))
        else:
            self.setValue(ctx, "{:s}".format(mod))

    def exitModification_assignment(self, ctx):
        expr = self.getValue(ctx.expression())
        self.setValue(ctx, "= {:s}".format(expr))

    def exitModification_assignment2(self, ctx):
        # TODO how to handle := operator?, is it just assignment
        expr = self.getValue(ctx.expression())
        self.setValue(ctx, "= {:s}".format(expr))

    def exitDeclaration(self, ctx):
        # set above based on component type
        self.setValue(ctx, "")

    def exitComment(self, ctx):
        self.setValue(ctx, "# {:s}".format(ctx.getText()))

    def exitString_comment(self, ctx):
        self.setValue(ctx, "# {:s}".format(ctx.getText()))

    def exitComponent_declaration(self, ctx):
        # set above based on component type
        self.setValue(ctx, "")

    def exitComponent_list(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitElement_modification(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitElement_modification_or_replaceable(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitType_prefix(self, ctx):
        self.setValue(ctx, "")

    def exitArgument(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitArgument_list(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitClass_modification(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitComponent_reference(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitFunction_arguments(self, ctx):
        if ctx.function_argument() is not None:
            s = ""
            args = ctx.function_argument()
            n = len(args)
            for i in range(n):
                s += "{:s}".format(self.getValue(args[i]))
                if i < n-1:
                    s += ", "
            self.setValue(ctx, s)
        elif ctx.named_arguments() is not None:
            args = self.getValue(ctx.named_arguments())
            self.setValue(ctx, args)

    def exitArguement_function(self, ctx):
        name = self.getValue(ctx.name())
        if ctx.named_arguments() is not None:
            named_arguments = self.getValue(ctx.named_arguements())
        else:
            named_arguments = ""
        self.setValue(ctx, "{name:s}({named_arguments:s})")

    def exitArgument_expression(self, ctx):
        expr = self.getValue(ctx.expression())
        self.setValue(ctx, expr)

    def exitFunction_call_args(self, ctx):
        args = ctx.function_arguments()
        if  args is not None:
            self.setValue(ctx, self.getValue(args))
        else:
            self.setValue(ctx, "")

    def exitClass_specifier(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitClass_definition(self, ctx):
        # TODO
        self.setValue(ctx, "")

    def exitClass_stored_definition(self, ctx):
        # TODO
        self.setValue(ctx, "")

    #-------------------------------------------------------------------------
    # Equation
    #-------------------------------------------------------------------------

    def exitEquation(self, ctx):
        self.setValue(ctx, self.getValue(ctx.equation_options()))

    def exitEquation_simple(self, ctx):
        self.setValue(
            ctx,
            "{:s} - {:s}".format(
                self.getValue(ctx.simple_expression()),
                self.getValue(ctx.expression())))

    def exitEquation_if(self, ctx):
        raise NotImplementedError("")

    def exitEquation_for(self, ctx):
        raise NotImplementedError("")

    def exitEquation_connect_clause(self, ctx):
        raise NotImplementedError("")

    def exitEquation_function(self, ctx):
        self.setValue(ctx, "{name:s}({args:})".format(**{
            'name': self.getValue(ctx.name()),
            'args': self.getValue(ctx.function_call_args())
            }))

    def exitWhen_equation(self, ctx):
        exprs = ctx.expression()
        eqns = ctx.equation()
        t = jinja2.Template("""
if {{ walker.getValue(exprs[0]) -}}:
    {{ walker.getValue(eqns[0]) }}
{% for i in range(1, exprs|length) %}
elif {{ walker.getValue(exprs[i]) }}:
    {{ walker.getValue(eqns[i]) }}
{% endfor %}
""")
        # self.setValue(ctx, t.render({
            # 'walker': self,
            # 'exprs': exprs,
            # 'eqns': eqns,
            # }))
        self.setValue(ctx, '')

    def exitEquation_when(self, ctx):
        self.setValue(ctx, self.getValue(ctx.when_equation()))

    #-------------------------------------------------------------------------
    # Expression
    #-------------------------------------------------------------------------

    def exitExpression_simple(self, ctx):
        self.setValue(ctx, self.getValue(ctx.simple_expression()))

    def exitSimple_expression(self, ctx):
        # TODO, can have more than one expr
        self.setValue(ctx, self.getValue(ctx.expr()[0]))

    def exitExpr_primary(self, ctx):
        self.setValue(ctx, self.getValue(ctx.primary()))

    def exitExpr_neg(self, ctx):
        self.setValue(ctx, '-({:s})'.format(self.getValue(ctx.expr())))

    def exitExpr_rel(self, ctx):
        self.setValue(ctx, '({:s} {:s} {:s})'.format(
            self.getValue(ctx.expr()[0]),
            ctx.op.text,
            self.getValue(ctx.expr()[1])))

    def exitExpr_mul(self, ctx):
        self.setValue(ctx, '({:s} {:s} {:s})'.format(
            self.getValue(ctx.expr()[0]),
            ctx.op.text,
            self.getValue(ctx.expr()[1])))

    #-------------------------------------------------------------------------
    # Primary
    #-------------------------------------------------------------------------

    def exitPrimary_unsigned_number(self, ctx):
        self.setValue(ctx, ctx.getText())

    def exitPrimary_component_reference(self, ctx):
        self.setValue(ctx, ctx.getText())

    def exitPrimary_derivative(self, ctx):
        name = ctx.function_call_args().function_arguments().function_argument()[0].getText()
        self.setValue(ctx, '{:s}.diff(self.t)'.format(name))

    def exitPrimary_string(self, ctx):
        self.setValue(ctx, ctx.getText())

    def exitPrimary_false(self, ctx):
        self.setValue(ctx, ctx.getText())


def generate(modelica_model, trace=False):
    "The modelica model"
    input_stream = antlr4.InputStream(modelica_model)
    lexer = ModelicaLexer(input_stream)
    stream = antlr4.CommonTokenStream(lexer)
    parser = ModelicaParser(stream)
    tree = parser.stored_definition()
    # print(tree.toStringTree(recog=parser))
    sympyPrinter = SympyPrinter(parser, trace)
    walker = antlr4.ParseTreeWalker()
    walker.walk(sympyPrinter, tree)
    return sympyPrinter.result


def main(argv):
    #pylint: disable=unused-argument
    "The main function"
    parser = argparse.ArgumentParser()
    parser.add_argument('src')
    parser.add_argument('out')
    parser.add_argument('-t', '--trace', action='store_true')
    parser.set_defaults(trace=False)
    args = parser.parse_args(argv)
    with open(args.src, 'r') as f:
        modelica_model = f.read()
    sympy_model = generate(modelica_model, trace=args.trace)

    with open(args.out, 'w') as f:
        f.write(sympy_model)

if __name__ == '__main__':
    main(sys.argv[1:])

# vim: set et ft=python fenc=utf-8 ff=unix sts=0 sw=4 ts=4 :



#!/usr/bin/env python
"""A python/modelica based simulation environment.

Pymola contains a Python based compiler for the modelica language
and enables interacting with Modelica easily in Python.

"""

from __future__ import print_function
import os
import sys
import subprocess

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py
from setuptools.command.build_ext import build_ext
from setuptools import Command

MAJOR = 0
MINOR = 0
MICRO = 6
ISRELEASED = True
VERSION = '%d.%d.%d' % (MAJOR, MINOR, MICRO)
DOCLINES = __doc__.split("\n")

CLASSIFIERS = """\
Development Status :: 1 - Planning
Intended Audience :: Science/Research
Intended Audience :: Developers
License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)
Programming Language :: Python
Programming Language :: Python :: 3
Programming Language :: Other
Topic :: Software Development
Topic :: Scientific/Engineering :: Artificial Intelligence
Topic :: Scientific/Engineering :: Mathematics
Topic :: Scientific/Engineering :: Physics
Topic :: Scientific/Engineering :: Visualization
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: Unix
Operating System :: MacOS
Topic :: Software Development :: Code Generators
Topic :: Software Development :: Compilers
Topic :: Software Development :: Embedded Systems
"""

#pylint: disable=no-init, too-few-public-methods

ROOT_DIR = os.path.dirname(os.path.realpath(__file__))

class AntlrBuildCommand(Command):
    """Customized setuptools build command."""
    user_options=[]
    def initialize_options(self):
        pass
    def finalize_options(self):
        pass
    def run(self):
        "Run the build command"
        call_antlr4('Modelica.g4')

def call_antlr4(arg):
    "calls antlr4 on grammar file"
    #pylint: disable=unused-argument, unused-variable
    antlr_path = os.path.join(ROOT_DIR, "java", "antlr-4.5.1-complete.jar")
    classpath = ".:{:s}:$CLASSPATH".format(antlr_path)
    generated = os.path.join(ROOT_DIR, 'pymola', 'generated')
    cmd = "java -Xmx500M -cp \"{classpath:s}\" org.antlr.v4.Tool {arg:s}" \
            " -o {generated:s} -Dlanguage=Python2".format(**locals())
    print(cmd)
    proc = subprocess.Popen(cmd.split(), cwd=os.path.join(ROOT_DIR, 'pymola'))
    proc.communicate()
    with open(os.path.join(ROOT_DIR, 'pymola', 'generated', '__init__.py'), 'w') as fid:
        fid.write('')

def git_version():
    "Return the git revision as a string"
    def _minimal_ext_cmd(cmd):
        "construct minimal environment"
        env = {}
        for k in ['SYSTEMROOT', 'PATH']:
            var = os.environ.get(k)
            if var is not None:
                env[k] = var
        # LANGUAGE is used on win32
        env['LANGUAGE'] = 'C'
        env['LANG'] = 'C'
        env['LC_ALL'] = 'C'
        out = subprocess.Popen(
            cmd,
            stdout=subprocess.PIPE,
            env=env).communicate()[0]
        return out

    try:
        out = _minimal_ext_cmd(['git', 'rev-parse', 'HEAD'])
        git_revision = out.strip().decode('ascii')
    except OSError:
        git_revision = "Unknown"

    return git_revision


def get_version_info():
    """
    Adding the git rev number needs to be done inside write_version_py(),
    otherwise the import of package.version messes up
    the build under Python 3.
    """
    full_version = VERSION
    if os.path.exists('.git'):
        git_revision = git_version()
    elif os.path.exists('pymola/version.py'):
        # must be a source distribution, use existing version file
        try:
            from pymola.version import git_revision as git_revision
        except ImportError:
            raise ImportError("Unable to import git_revision. Try removing "
                              "pymola/version.py and the build directory "
                              "before building.")
    else:
        git_revision = "Unknown"

    if not ISRELEASED:
        full_version += '.git.' + git_revision[:7]

    return full_version, git_revision


def write_version_py(filename='pymola/version.py'):
    """
    Create a version.py file.
    """
    cnt = """
# THIS FILE IS GENERATED FROM SETUP.PY
short_version = '%(version)s'
version = '%(version)s'
full_version = '%(full_version)s'
git_revision = '%(git_revision)s'
release = %(isrelease)s

if not release:
    version = full_version
"""
    full_version, git_revision = get_version_info()

    with open(filename, 'w') as fid:
        try:
            fid.write(cnt % {'version': VERSION,
                             'full_version': full_version,
                             'git_revision': git_revision,
                             'isrelease': str(ISRELEASED)})
        finally:
            fid.close()


def setup_package():
    """
    Setup the package.
    """
    src_path = os.path.dirname(os.path.abspath(sys.argv[0]))
    old_path = os.getcwd()
    os.chdir(src_path)
    sys.path.insert(0, src_path)

    # Rewrite the version file everytime
    write_version_py()

    metadata = dict(
        name='pymola',
        maintainer="James Goppert",
        maintainer_email="james.goppert@gmail.com",
        description=DOCLINES[0],
        long_description="\n".join(DOCLINES[2:]),
        url='https://github.com/jgoppert/pymola',
        author='James Goppert',
        author_email='james.goppert@gmail.com',
        download_url='https://github.com/jgoppert/pymola',
        license='GPLv3+',
        classifiers=[_f for _f in CLASSIFIERS.split('\n') if _f],
        platforms=["Windows", "Linux", "Solaris", "Mac OS-X", "Unix"],
        install_requires=['antlr4-python2-runtime', 'jinja2'],
        tests_require=['nose'],
        test_suite='nose.collector',
        packages=find_packages(
            # choosing to distribute tests
            # exclude=['*.test']
        ),
        cmdclass={
            'antlr': AntlrBuildCommand,
        }
    )

    full_version = get_version_info()[0]
    metadata['version'] = full_version

    try:
        setup(**metadata)
    finally:
        del sys.path[0]
        os.chdir(old_path)
    return

if __name__ == '__main__':
    setup_package()
